@@ -1,7 +1,7 @@
 
 #==============================================================================#
 # exerb/archive.rb
-# $Id: archive.rb,v 1.11 2003/02/21 05:14:48 yuya Exp $
+# $Id: archive.rb,v 1.12 2003/05/06 04:14:05 yuya Exp $
 #==============================================================================#
 
 #==============================================================================#
@@ -30,7 +30,11 @@ module Exerb
     end
 
     def add_binary(name, filepath)
-      @entries << EntryFile.new(name, ArchiveHeader::ENTRY_TYPE_BINARY, filepath)
+      if /\.rb$/i =~ filepath
+        @entries << BinaryEntryFile.new(name.sub(/\.rb$/i, '.brb'), ArchiveHeader::ENTRY_TYPE_BINARY, filepath)
+      else
+        @entries << EntryFile.new(name, ArchiveHeader::ENTRY_TYPE_BINARY, filepath)
+      end
     end
 
     def pack
@@ -91,6 +95,17 @@ module Exerb
 
     end # EntryFile
 
+    #==========================================================================#
+    # BinaryEntryFile Class
+    class BinaryEntryFile < EntryFile
+
+      def pack
+        require 'bruby'
+        return BRb.dumpfile(filepath)
+      end
+
+    end # EntryFile
+
   end # Archive
 
 end # Exerb
