@@ -1,4 +1,4 @@
-// $Id: exerb.cpp,v 1.99 2003/02/20 09:18:47 yuya Exp $
+// $Id: exerb.cpp,v 1.101 2003/03/07 06:00:58 yuya Exp $
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -21,7 +21,6 @@ static VALUE ExExecuteStartupScript();
 #endif
 static void  ExCleanup();
 static void  ExFinalRuby();
-static int   ExFailed();
 static void  ExOnErrorID(int resource_id);
 static void  ExCreateTemporaryFile(char *filepath_buffer);
 static void  ExWriteFile(char *filepath, void *buffer, DWORD size);
@@ -239,7 +238,17 @@ ExExecute()
 	::rb_protect(ExExecuteStartupScript, 0, &state);
 
 	if ( state ) {
-		return ::ExFailed();
+		if ( !ruby_errinfo ) {
+			return 1;
+		}
+
+		if ( ::rb_obj_is_kind_of(ruby_errinfo, rb_eSystemExit) ) {
+			return NUM2INT(::rb_iv_get(ruby_errinfo, "status"));
+		}
+
+		::ExOnFail(ruby_errinfo);
+
+		return 1;
 	} else {
 		return 0;
 	}
@@ -294,22 +303,6 @@ ExFinalRuby()
 	::ruby_finalize();
 }
 
-static int
-ExFailed()
-{
-	if ( !ruby_errinfo ) {
-		return 1;
-	}
-
-	if ( ::rb_obj_is_kind_of(ruby_errinfo, rb_eSystemExit) == Qtrue ) {
-		return 0;
-	}
-
-	::ExOnFail(ruby_errinfo);
-
-	return 1;
-}
-
 static void
 ExOnErrorID(int resource_id)
 {
