@@ -1,4 +1,4 @@
-// $Id: exerb.h,v 1.15 2003/01/30 07:43:50 yuya Exp $
+// $Id: exerb.h,v 1.17 2003/02/21 05:30:13 yuya Exp $
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -58,5 +58,7 @@ typedef void (*ON_FAIL_FNC)(VALUE);
 int  ExMain(int argc, char **argv, ON_ERROR_FNC onerror, ON_FAIL_FNC onfail);
 void ExGetSelfFilePath(LPSTR filepath_buffer, UINT size);
 void ExGetSelfFileName(LPSTR filename_buffer, UINT size);
+void ExRaiseException(VALUE klass, int res_id);
+void ExRaiseExceptionVar(VALUE klass, int res_id, ...);
 
 ////////////////////////////////////////////////////////////////////////////////
