@@ -3,7 +3,7 @@
   hash.c -
 
   $Author: matz $
-  $Date: 2003/02/04 07:27:41 $
+  $Date: 2003/08/01 20:16:48 $
   created at: Mon Nov 22 18:51:18 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -16,6 +16,7 @@
 #include "st.h"
 #include "util.h"
 #include "rubysig.h"
+#include "version.h"
 
 #ifdef __APPLE__
 #include <crt_externs.h>
@@ -44,7 +45,7 @@ rb_hash_freeze(hash)
 VALUE rb_cHash;
 
 static VALUE envtbl;
-static ID id_hash, id_yield, id_default;
+static ID id_hash, id_call, id_default;
 
 VALUE
 rb_hash(obj)
@@ -65,6 +66,8 @@ rb_any_cmp(a, b)
     VALUE a, b;
 {
     VALUE args[2];
+
+    if (a == b) return 0;
     if (FIXNUM_P(a) && FIXNUM_P(b)) {
 	return a != b;
     }
@@ -72,10 +75,10 @@ rb_any_cmp(a, b)
 	TYPE(b) == T_STRING && RBASIC(b)->klass == rb_cString) {
 	return rb_str_cmp(a, b);
     }
+    if (a == Qundef || b == Qundef) return -1;
     if (SYMBOL_P(a) && SYMBOL_P(b)) {
 	return a != b;
     }
-    if (a == Qundef || b == Qundef) return -1;
 
     args[0] = a;
     args[1] = b;
@@ -99,12 +102,10 @@ rb_any_hash(a)
 	break;
 
       default:
-	DEFER_INTS;
 	hval = rb_funcall(a, id_hash, 0);
 	if (!FIXNUM_P(hval)) {
 	    hval = rb_funcall(hval, '%', 1, INT2FIX(536870923));
 	}
-	ENABLE_INTS;
 	return (int)FIX2LONG(hval);
     }
 }
@@ -208,7 +209,7 @@ rb_hash_initialize(argc, argv, hash)
 	if (argc > 0) {
 	    rb_raise(rb_eArgError, "wrong number of arguments");
 	}
-	RHASH(hash)->ifnone = rb_f_lambda();
+	RHASH(hash)->ifnone = rb_block_proc();
 	FL_SET(hash, HASH_PROC_DEFAULT);
     }
     else {
@@ -328,7 +329,7 @@ rb_hash_default(argc, argv, hash)
 
     rb_scan_args(argc, argv, "01", &key);
     if (FL_TEST(hash, HASH_PROC_DEFAULT)) {
-	return rb_funcall(RHASH(hash)->ifnone, id_yield, 2, hash, key);
+	return rb_funcall(RHASH(hash)->ifnone, id_call, 2, hash, key);
     }
     return RHASH(hash)->ifnone;
 }
@@ -388,7 +389,7 @@ rb_hash_indexes(argc, argv, hash)
     VALUE indexes;
     int i;
 
-    rb_warn("Hash#%s is deprecated; use Hash#select",
+    rb_warn("Hash#%s is deprecated; use Hash#values_at",
 	    rb_id2name(rb_frame_last_func()));
     indexes = rb_ary_new2(argc);
     for (i=0; i<argc; i++) {
@@ -406,12 +407,12 @@ rb_hash_delete(hash, key)
 
     rb_hash_modify(hash);
     if (RHASH(hash)->iter_lev > 0) {
-	if (st_delete_safe(RHASH(hash)->tbl, &key, &val, Qundef)) {
+	if (st_delete_safe(RHASH(hash)->tbl, (st_data_t*)&key, &val, Qundef)) {
 	    FL_SET(hash, HASH_DELETED);
 	    return val;
 	}
     }
-    else if (st_delete(RHASH(hash)->tbl, &key, &val))
+    else if (st_delete(RHASH(hash)->tbl, (st_data_t*)&key, &val))
 	return val;
     if (rb_block_given_p()) {
 	return rb_yield(key);
@@ -452,7 +453,7 @@ rb_hash_shift(hash)
 	return rb_assoc_new(var.key, var.val);
     }
     else if (FL_TEST(hash, HASH_PROC_DEFAULT)) {
-	return rb_funcall(RHASH(hash)->ifnone, id_yield, 2, hash, Qnil);
+	return rb_funcall(RHASH(hash)->ifnone, id_call, 2, hash, Qnil);
     }
     else {
 	return RHASH(hash)->ifnone;
@@ -464,7 +465,7 @@ delete_if_i(key, value)
     VALUE key, value;
 {
     if (key == Qundef) return ST_CONTINUE;
-    if (RTEST(rb_yield(rb_assoc_new(key, value))))
+    if (RTEST(rb_yield_values(2, key, value)))
 	return ST_DELETE;
     return ST_CONTINUE;
 }
@@ -499,35 +500,46 @@ static enum st_retval
 select_i(key, value, result)
     VALUE key, value, result;
 {
-    VALUE assoc;
-
     if (key == Qundef) return ST_CONTINUE;
-    assoc = rb_assoc_new(key, value);
-    if (RTEST(rb_yield(assoc)))
-	rb_ary_push(result, assoc);
+    if (RTEST(rb_yield_values(2, key, value)))
+	rb_ary_push(result, rb_assoc_new(key, value));
     return ST_CONTINUE;
 }
 
+VALUE
+rb_hash_values_at(argc, argv, hash)
+    int argc;
+    VALUE *argv;
+    VALUE hash;
+{
+    VALUE result = rb_ary_new();
+    long i;
+
+    for (i=0; i<argc; i++) {
+	rb_ary_push(result, rb_hash_aref(hash, argv[i]));
+    }
+    return result;
+}
+
 VALUE
 rb_hash_select(argc, argv, hash)
     int argc;
     VALUE *argv;
     VALUE hash;
 {
-    VALUE result = rb_ary_new();
-    long i;
+    VALUE result;
 
-    if (rb_block_given_p()) {
-	if (argc > 0) {
-	    rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
-	}
-	rb_hash_foreach(hash, select_i, result);
+    if (!rb_block_given_p()) {
+#if RUBY_VERSION_CODE < 181
+	rb_warn("Hash#select(key..) is deprecated; use Hash#values_at");
+#endif
+	return rb_hash_values_at(argc, argv, hash);
     }
-    else {
-	for (i=0; i<argc; i++) {
-	    rb_ary_push(result, rb_hash_aref(hash, argv[i]));
-	}
+    if (argc > 0) {
+	rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
     }
+    result = rb_ary_new();
+    rb_hash_foreach(hash, select_i, result);
     return result;
 }
 
@@ -647,7 +659,7 @@ each_pair_i(key, value)
     VALUE key, value;
 {
     if (key == Qundef) return ST_CONTINUE;
-    rb_yield(rb_assoc_new(key, value));
+    rb_yield_values(2, key, value);
     return ST_CONTINUE;
 }
 
@@ -922,7 +934,7 @@ rb_hash_update_block_i(key, value, hash)
 {
     if (key == Qundef) return ST_CONTINUE;
     if (rb_hash_has_key(hash, key)) {
-	value = rb_yield(rb_ary_new3(3, key, rb_hash_aref(hash, key), value));
+	value = rb_yield_values(3, key, rb_hash_aref(hash, key), value);
     }
     rb_hash_aset(hash, key, value);
     return ST_CONTINUE;
@@ -969,6 +981,25 @@ extern char **environ;
 #define FREE_ENVIRON(e)
 #endif
 
+static VALUE
+env_str_new(ptr, len)
+    const char *ptr;
+    long len;
+{
+    VALUE str = rb_tainted_str_new(ptr, len);
+
+    rb_obj_freeze(str);
+    return str;
+}
+
+static VALUE
+env_str_new2(ptr)
+    const char *ptr;
+{
+    if (!ptr) return Qnil;
+    return env_str_new(ptr, strlen(ptr));
+}
+
 static VALUE
 env_delete(obj, name)
     VALUE obj, name;
@@ -983,14 +1014,15 @@ env_delete(obj, name)
     }
     val = getenv(nam);
     if (val) {
-	VALUE value = rb_tainted_str_new2(val);
+	VALUE value = env_str_new2(val);
 
 	ruby_setenv(nam, 0);
-#ifdef __BORLANDC__
-	if (strcmpi(nam, "PATH") == 0) {
+#ifdef ENV_IGNORECASE
+	if (strcasecmp(nam, PATH_ENV) == 0)
 #else
-	if (strcmp(nam, "PATH") == 0) {
+	if (strcmp(nam, PATH_ENV) == 0)
 #endif
+	{
 	    path_tainted = 0;
 	}
 	return value;
@@ -1002,8 +1034,10 @@ static VALUE
 env_delete_m(obj, name)
     VALUE obj, name;
 {
-    VALUE val = env_delete(obj, name);
-    if (rb_block_given_p()) rb_yield(name);
+    VALUE val;
+
+    val = env_delete(obj, name);
+    if (NIL_P(val) && rb_block_given_p()) rb_yield(name);
     return val;
 }
 
@@ -1020,13 +1054,18 @@ rb_f_getenv(obj, name)
     }
     env = getenv(nam);
     if (env) {
-#ifdef __BORLANDC__
-	if (strcmpi(nam, "PATH") == 0 && !rb_env_path_tainted())
+#ifdef ENV_IGNORECASE
+	if (strcasecmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #else
-	if (strcmp(nam, "PATH") == 0 && !rb_env_path_tainted())
+	if (strcmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #endif
-	    return rb_str_new2(env);
-	return rb_tainted_str_new2(env);
+	{
+	    VALUE str = rb_str_new2(env);
+
+	    rb_obj_freeze(str);
+	    return str;
+	}
+	return env_str_new2(env);
     }
     return Qnil;
 }
@@ -1058,13 +1097,13 @@ env_fetch(argc, argv)
 	}
 	return if_none;
     }
-#ifdef __BORLANDC__
-    if (strcmpi(nam, "PATH") == 0 && !rb_env_path_tainted())
+#ifdef ENV_IGNORECASE
+    if (strcasecmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #else
-    if (strcmp(nam, "PATH") == 0 && !rb_env_path_tainted())
+    if (strcmp(nam, PATH_ENV) == 0 && !rb_env_path_tainted())
 #endif
 	return rb_str_new2(env);
-    return rb_tainted_str_new2(env);
+    return env_str_new2(env);
 }
 
 static void
@@ -1078,7 +1117,7 @@ int
 rb_env_path_tainted()
 {
     if (path_tainted < 0) {
-	path_tainted_p(getenv("PATH"));
+	path_tainted_p(getenv(PATH_ENV));
     }
     return path_tainted;
 }
@@ -1093,8 +1132,8 @@ envix(nam)
     env = GET_ENVIRON(environ);
     for (i = 0; env[i]; i++) {
 	if (
-#ifdef WIN32
-	    strnicmp(env[i],nam,len) == 0
+#ifdef ENV_IGNORECASE
+	    strncasecmp(env[i],nam,len) == 0
 #else
 	    memcmp(env[i],nam,len) == 0
 #endif
@@ -1110,7 +1149,7 @@ ruby_setenv(name, value)
     const char *name;
     const char *value;
 {
-#if defined(WIN32) && !defined(__CYGWIN32__)
+#if defined(_WIN32)
     /* The sane way to deal with the environment.
      * Has these advantages over putenv() & co.:
      *  * enables us to store a truly empty value in the
@@ -1221,7 +1260,11 @@ env_aset(obj, nm, val)
 	rb_raise(rb_eArgError, "bad environment variable value");
 
     ruby_setenv(name, value);
-    if (strcmp(name, "PATH") == 0) {
+#ifdef ENV_IGNORECASE
+    if (strcasecmp(name, PATH_ENV) == 0) {
+#else
+    if (strcmp(name, PATH_ENV) == 0) {
+#endif
 	if (OBJ_TAINTED(val)) {
 	    /* already tainted, no check */
 	    path_tainted = 1;
@@ -1244,7 +1287,7 @@ env_keys()
     while (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    rb_ary_push(ary, rb_tainted_str_new(*env, s-*env));
+	    rb_ary_push(ary, env_str_new(*env, s-*env));
 	}
 	env++;
     }
@@ -1253,21 +1296,16 @@ env_keys()
 }
 
 static VALUE
-env_each_key(hash)
-    VALUE hash;
+env_each_key(ehash)
+    VALUE ehash;
 {
-    char **env;
+    VALUE keys = env_keys();
+    long i;
 
-    env = GET_ENVIRON(environ);
-    while (*env) {
-	char *s = strchr(*env, '=');
-	if (s) {
-	    rb_yield(rb_tainted_str_new(*env, s-*env));
-	}
-	env++;
+    for (i=0; i<RARRAY(keys)->len; i++) {
+	rb_yield(RARRAY(keys)->ptr[i]);
     }
-    FREE_ENVIRON(environ);
-    return Qnil;
+    return ehash;
 }
 
 static VALUE
@@ -1280,7 +1318,7 @@ env_values()
     while (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    rb_ary_push(ary, rb_tainted_str_new2(s+1));
+	    rb_ary_push(ary, env_str_new2(s+1));
 	}
 	env++;
     }
@@ -1289,64 +1327,62 @@ env_values()
 }
 
 static VALUE
-env_each_value(hash)
-    VALUE hash;
+env_each_value(ehash)
+    VALUE ehash;
 {
-    char **env;
+    VALUE values = env_values();
+    long i;
 
-    env = GET_ENVIRON(environ);
-    while (*env) {
-	char *s = strchr(*env, '=');
-	if (s) {
-	    rb_yield(rb_tainted_str_new2(s+1));
-	}
-	env++;
+    for (i=0; i<RARRAY(values)->len; i++) {
+	rb_yield(RARRAY(values)->ptr[i]);
     }
-    FREE_ENVIRON(environ);
-    return Qnil;
+    return ehash;
 }
 
 static VALUE
-env_each(hash)
-    VALUE hash;
+env_each(ehash)
+    VALUE ehash;
 {
     char **env;
+    VALUE ary = rb_ary_new();
+    long i;
 
     env = GET_ENVIRON(environ);
     while (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    rb_yield(rb_assoc_new(rb_tainted_str_new(*env, s-*env),
-				  rb_tainted_str_new2(s+1)));
+	    rb_ary_push(ary, env_str_new(*env, s-*env));
+	    rb_ary_push(ary, env_str_new2(s+1));
 	}
 	env++;
     }
     FREE_ENVIRON(environ);
-    return Qnil;
+
+    for (i=0; i<RARRAY(ary)->len; i+=2) {
+	rb_yield_values(2, RARRAY(ary)->ptr[i], RARRAY(ary)->ptr[i+1]);
+    }
+    return ehash;
 }
 
 static VALUE
 env_reject_bang()
 {
     volatile VALUE keys;
-    VALUE *ptr;
-    long len;
+    long i;
     int del = 0;
 
     rb_secure(4);
     keys = env_keys();
-    ptr = RARRAY(keys)->ptr;
-    len = RARRAY(keys)->len; 
 
-    while (len--) {
-	VALUE val = rb_f_getenv(Qnil, *ptr);
+    for (i=0; i<RARRAY(keys)->len; i++) {
+	VALUE val = rb_f_getenv(Qnil, RARRAY(keys)->ptr[i]);
 	if (!NIL_P(val)) {
-	    if (RTEST(rb_yield(rb_assoc_new(*ptr, val)))) {
-		env_delete(Qnil, *ptr);
+	    if (RTEST(rb_yield_values(2, RARRAY(keys)->ptr[i], val))) {
+		FL_UNSET(RARRAY(keys)->ptr[i], FL_TAINT);
+		env_delete(Qnil, RARRAY(keys)->ptr[i]);
 		del++;
 	    }
 	}
-	ptr++;
     }
     if (del == 0) return Qnil;
     return envtbl;
@@ -1360,38 +1396,49 @@ env_delete_if()
 }
 
 static VALUE
-env_select(argc, argv)
+env_values_at(argc, argv)
     int argc;
     VALUE *argv;
 {
     VALUE result = rb_ary_new();
     long i;
 
-    if (rb_block_given_p()) {
-	char **env;
+    for (i=0; i<argc; i++) {
+	rb_ary_push(result, rb_f_getenv(Qnil, argv[i]));
+    }
+    return result;
+}
 
-	if (argc > 0) {
-	    rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
-	}
-	env = GET_ENVIRON(environ);
-	while (*env) {
-	    char *s = strchr(*env, '=');
-	    if (s) {
-                VALUE assoc = rb_assoc_new(rb_tainted_str_new(*env, s-*env),
-                                           rb_tainted_str_new2(s+1));
-		if (RTEST(rb_yield(assoc))) {
-		    rb_ary_push(result, assoc);
-		}
+static VALUE
+env_select(argc, argv)
+    int argc;
+    VALUE *argv;
+{
+    VALUE result;
+    char **env;
+
+    if (!rb_block_given_p()) {
+	rb_warn("ENV.select(index..) is deprecated; use ENV.values_at");
+	return env_values_at(argc, argv);
+    }
+    if (argc > 0) {
+	rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
+    }
+    result = rb_ary_new();
+    env = GET_ENVIRON(environ);
+    while (*env) {
+	char *s = strchr(*env, '=');
+	if (s) {
+	    VALUE k = env_str_new(*env, s-*env);
+	    VALUE v = env_str_new2(s+1);
+	    if (RTEST(rb_yield_values(2, k, v))) {
+		rb_ary_push(result, rb_assoc_new(k, v));
 	    }
-	    env++;
-	}
-	FREE_ENVIRON(environ);
-    }
-    else {
-	for (i=0; i<argc; i++) {
-	    rb_ary_push(result, rb_f_getenv(Qnil, argv[i]));
 	}
+	env++;
     }
+    FREE_ENVIRON(environ);
+
     return result;
 }
 
@@ -1399,20 +1446,16 @@ static VALUE
 env_clear()
 {
     volatile VALUE keys;
-    VALUE *ptr;
-    long len;
+    long i;
     
     rb_secure(4);
     keys = env_keys();
-    ptr = RARRAY(keys)->ptr;
-    len = RARRAY(keys)->len; 
 
-    while (len--) {
-	VALUE val = rb_f_getenv(Qnil, *ptr);
+    for (i=0; i<RARRAY(keys)->len; i++) {
+	VALUE val = rb_f_getenv(Qnil, RARRAY(keys)->ptr[i]);
 	if (!NIL_P(val)) {
-	    env_delete(Qnil, *ptr);
+	    env_delete(Qnil, RARRAY(keys)->ptr[i]);
 	}
-	ptr++;
     }
     return envtbl;
 }
@@ -1463,8 +1506,8 @@ env_to_a()
     while (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    rb_ary_push(ary, rb_assoc_new(rb_tainted_str_new(*env, s-*env),
-					  rb_tainted_str_new2(s+1)));
+	    rb_ary_push(ary, rb_assoc_new(env_str_new(*env, s-*env),
+					  env_str_new2(s+1)));
 	}
 	env++;
     }
@@ -1527,9 +1570,13 @@ env_has_value(dmy, value)
     if (TYPE(value) != T_STRING) return Qfalse;
     env = GET_ENVIRON(environ);
     while (*env) {
-	char *s = strchr(*env, '=')+1;
-	if (s) {
+	char *s = strchr(*env, '=');
+	if (s++) {
+#ifdef ENV_IGNORECASE
+	    if (strncasecmp(s, RSTRING(value)->ptr, strlen(s)) == 0) {
+#else
 	    if (strncmp(s, RSTRING(value)->ptr, strlen(s)) == 0) {
+#endif
 		FREE_ENVIRON(environ);
 		return Qtrue;
 	    }
@@ -1547,13 +1594,17 @@ env_index(dmy, value)
     char **env;
     VALUE str;
 
-    if (TYPE(value) != T_STRING) return Qnil;
+    StringValue(value);
     env = GET_ENVIRON(environ);
     while (*env) {
-	char *s = strchr(*env, '=')+1;
-	if (s) {
+	char *s = strchr(*env, '=');
+	if (s++) {
+#ifdef ENV_IGNORECASE
+	    if (strncasecmp(s, RSTRING(value)->ptr, strlen(s)) == 0) {
+#else
 	    if (strncmp(s, RSTRING(value)->ptr, strlen(s)) == 0) {
-		str = rb_tainted_str_new(*env, s-*env-1);
+#endif
+		str = env_str_new(*env, s-*env-1);
 		FREE_ENVIRON(environ);
 		return str;
 	    }
@@ -1572,18 +1623,15 @@ env_indexes(argc, argv)
     int i;
     VALUE indexes = rb_ary_new2(argc);
 
-    rb_warn("ENV.%s is deprecated; use ENV.select",
+    rb_warn("ENV.%s is deprecated; use ENV.values_at",
 	    rb_id2name(rb_frame_last_func()));
     for (i=0;i<argc;i++) {
-	char *v = 0;
-	if (TYPE(argv[i]) == T_STRING) {
-	    v = getenv(RSTRING(argv[i])->ptr);
-	}
-	if (v) {
-	    RARRAY(indexes)->ptr[i] = rb_tainted_str_new2(v);
+	VALUE tmp = rb_check_string_type(argv[i]);
+	if (NIL_P(tmp)) {
+	    RARRAY(indexes)->ptr[i] = Qnil;
 	}
 	else {
-	    RARRAY(indexes)->ptr[i] = Qnil;
+	    RARRAY(indexes)->ptr[i] = env_str_new2(getenv(RSTRING(tmp)->ptr));
 	}
 	RARRAY(indexes)->len = i+1;
     }
@@ -1601,8 +1649,8 @@ env_to_hash()
     while (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    rb_hash_aset(hash, rb_tainted_str_new(*env, s-*env),
-			       rb_tainted_str_new2(s+1));
+	    rb_hash_aset(hash, env_str_new(*env, s-*env),
+			       env_str_new2(s+1));
 	}
 	env++;
     }
@@ -1625,8 +1673,8 @@ env_shift()
     if (*env) {
 	char *s = strchr(*env, '=');
 	if (s) {
-	    VALUE key = rb_tainted_str_new(*env, s-*env);
-	    VALUE val = rb_tainted_str_new2(getenv(RSTRING(key)->ptr));
+	    VALUE key = env_str_new(*env, s-*env);
+	    VALUE val = env_str_new2(getenv(RSTRING(key)->ptr));
 	    env_delete(Qnil, key);
 	    return rb_assoc_new(key, val);
 	}
@@ -1659,18 +1707,14 @@ env_replace(env, hash)
     VALUE env, hash;
 {
     volatile VALUE keys = env_keys();
-    VALUE *ptr;
-    long len;
+    long i;
 
     if (env == hash) return env;
     hash = to_hash(hash);
     st_foreach(RHASH(hash)->tbl, env_replace_i, keys);
 
-    ptr = RARRAY(keys)->ptr;
-    len = RARRAY(keys)->len; 
-
-    while (len--) {
-	env_delete(env, *ptr++);
+    for (i=0; i<RARRAY(keys)->len; i++) {
+	env_delete(env, RARRAY(keys)->ptr[i]);
     }
     return env;
 }
@@ -1681,7 +1725,7 @@ env_update_i(key, val)
 {
     if (key != Qundef) {
 	if (rb_block_given_p()) {
-	    val = rb_yield(rb_ary_new3(3, key, rb_f_getenv(Qnil, key), val));
+	    val = rb_yield_values(3, key, rb_f_getenv(Qnil, key), val);
 	}
 	env_aset(Qnil, key, val);
     }
@@ -1702,7 +1746,7 @@ void
 Init_Hash()
 {
     id_hash = rb_intern("hash");
-    id_yield = rb_intern("yield");
+    id_call = rb_intern("call");
     id_default = rb_intern("default");
 
     rb_cHash = rb_define_class("Hash", rb_cObject);
@@ -1712,8 +1756,7 @@ Init_Hash()
     rb_define_alloc_func(rb_cHash, hash_alloc);
     rb_define_singleton_method(rb_cHash, "[]", rb_hash_s_create, -1);
     rb_define_method(rb_cHash,"initialize", rb_hash_initialize, -1);
-
-    rb_define_method(rb_cHash,"copy_object", rb_hash_replace, 1);
+    rb_define_method(rb_cHash,"initialize_copy", rb_hash_replace, 1);
     rb_define_method(rb_cHash,"rehash", rb_hash_rehash, 0);
 
     rb_define_method(rb_cHash,"to_hash", rb_hash_to_hash, 0);
@@ -1744,6 +1787,7 @@ Init_Hash()
 
     rb_define_method(rb_cHash,"keys", rb_hash_keys, 0);
     rb_define_method(rb_cHash,"values", rb_hash_values, 0);
+    rb_define_method(rb_cHash,"values_at", rb_hash_values_at, -1);
 
     rb_define_method(rb_cHash,"shift", rb_hash_shift, 0);
     rb_define_method(rb_cHash,"delete", rb_hash_delete, 1);
@@ -1791,6 +1835,7 @@ Init_Hash()
     rb_define_singleton_method(envtbl,"inspect", env_inspect, 0);
     rb_define_singleton_method(envtbl,"rehash", env_none, 0);
     rb_define_singleton_method(envtbl,"to_a", env_to_a, 0);
+    rb_define_singleton_method(envtbl,"to_s", env_to_s, 0);
     rb_define_singleton_method(envtbl,"index", env_index, 1);
     rb_define_singleton_method(envtbl,"indexes", env_indexes, -1);
     rb_define_singleton_method(envtbl,"indices", env_indexes, -1);
@@ -1799,6 +1844,7 @@ Init_Hash()
     rb_define_singleton_method(envtbl,"empty?", env_empty_p, 0);
     rb_define_singleton_method(envtbl,"keys", env_keys, 0);
     rb_define_singleton_method(envtbl,"values", env_values, 0);
+    rb_define_singleton_method(envtbl,"values_at", env_values_at, -1);
     rb_define_singleton_method(envtbl,"include?", env_has_key, 1);
     rb_define_singleton_method(envtbl,"member?", env_has_key, 1);
     rb_define_singleton_method(envtbl,"has_key?", env_has_key, 1);
