@@ -502,8 +502,8 @@ static VALUE
 rb_reg_options_m(re)
     VALUE re;
 {
-    rb_reg_check(re);
-    return INT2NUM(RREGEXP(re)->ptr->options);
+    int options = rb_reg_options(re);
+    return INT2NUM(options);
 }
 
 static VALUE
@@ -582,7 +582,7 @@ match_alloc(klass)
 }
 
 static VALUE
-match_copy_object(obj, orig)
+match_init_copy(obj, orig)
     VALUE obj, orig;
 {
     if (obj == orig) return obj;
@@ -914,8 +914,9 @@ last_paren_match_getter()
 }
 
 static VALUE
-match_to_a(match)
+match_array(match, start)
     VALUE match;
+    int start;
 {
     struct re_registers *regs = RMATCH(match)->regs;
     VALUE ary = rb_ary_new2(regs->num_regs);
@@ -923,7 +924,7 @@ match_to_a(match)
     int i;
     int taint = OBJ_TAINTED(match);
     
-    for (i=0; i<regs->num_regs; i++) {
+    for (i=start; i<regs->num_regs; i++) {
 	if (regs->beg[i] == -1) {
 	    rb_ary_push(ary, Qnil);
 	}
@@ -936,6 +937,20 @@ match_to_a(match)
     return ary;
 }
 
+static VALUE
+match_to_a(match)
+    VALUE match;
+{
+    return match_array(match, 0);
+}
+
+static VALUE
+match_captures(match)
+    VALUE match;
+{
+    return match_array(match, 1);
+}
+
 static VALUE
 match_aref(argc, argv, match)
     int argc;
@@ -952,32 +967,53 @@ match_aref(argc, argv, match)
     return rb_reg_nth_match(FIX2INT(idx), match);
 }
 
+static VALUE match_entry _((VALUE, long));
+static VALUE
+match_entry(match, n)
+    VALUE match;
+    long n;
+{
+    return rb_reg_nth_match(n, match);
+}
+
+static VALUE
+match_values_at(argc, argv, match)
+    int argc;
+    VALUE *argv;
+    VALUE match;
+{
+    return rb_values_at(match, RMATCH(match)->regs->num_regs, argc, argv, match_entry);
+}
+
 static VALUE
 match_select(argc, argv, match)
     int argc;
     VALUE *argv;
     VALUE match;
 {
-    struct re_registers *regs = RMATCH(match)->regs;
-    VALUE target = RMATCH(match)->str;
-    VALUE result = rb_ary_new();
-    int i;
-    long idx;
-    int taint = OBJ_TAINTED(match);
+    if (!rb_block_given_p()) {
+	rb_warn("MatchData#select(index..) is deprecated; use MatchData#values_at");
+	return match_values_at(argc, argv, match);
+    }
+    if (argc > 0) {
+	rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
+    }
+    else {
+	struct re_registers *regs = RMATCH(match)->regs;
+	VALUE target = RMATCH(match)->str;
+	VALUE result = rb_ary_new();
+	int i;
+	int taint = OBJ_TAINTED(match);
 
-    for (i=0; i<argc; i++) {
-	idx = NUM2LONG(argv[i]);
-	if (idx < 0) idx += regs->num_regs;
-	if (idx < 0 || regs->num_regs <= idx) {
-	    rb_ary_push(result, Qnil);
-	}
-	else {
-	    VALUE str = rb_str_substr(target, regs->beg[idx], regs->end[idx]-regs->beg[idx]);
+	for (i=0; i<regs->num_regs; i++) {
+	    VALUE str = rb_str_substr(target, regs->beg[i], regs->end[i]-regs->beg[i]);
 	    if (taint) OBJ_TAINT(str);
-	    rb_ary_push(result, str);
+	    if (RTEST(rb_yield(str))) {
+		rb_ary_push(result, str);
+	    }
 	}
+	return result;
     }
-    return result;
 }
 
 static VALUE
@@ -1267,9 +1303,10 @@ rb_reg_initialize_m(argc, argv, self)
 	    if (FIXNUM_P(argv[1])) flags = FIX2INT(argv[1]);
 	    else if (RTEST(argv[1])) flags = RE_OPTION_IGNORECASE;
 	}
-	if (argc == 3) {
+	if (argc == 3 && !NIL_P(argv[2])) {
 	    char *kcode = StringValuePtr(argv[2]);
 
+	    flags &= ~0x70;
 	    switch (kcode[0]) {
 	      case 'n': case 'N':
 		flags |= 16;
@@ -1318,6 +1355,7 @@ rb_reg_quote(str)
 	  case '*': case '.': case '\\':
 	  case '?': case '+': case '^': case '$':
 	  case ' ': case '#':
+	  case '\t': case '\f': case '\n': case '\r':
 	    goto meta_found;
 	}
     }
@@ -1345,9 +1383,29 @@ rb_reg_quote(str)
 	  case '(': case ')': case '|': case '-':
 	  case '*': case '.': case '\\':
 	  case '?': case '+': case '^': case '$':
-	  case ' ': case '#':
+	  case '#':
 	    *t++ = '\\';
 	    break;
+	  case ' ':
+	    *t++ = '\\';
+	    *t++ = ' ';
+	    continue;
+	  case '\t':
+	    *t++ = '\\';
+	    *t++ = 't';
+	    continue;
+	  case '\n':
+	    *t++ = '\\';
+	    *t++ = 'n';
+	    continue;
+	  case '\r':
+	    *t++ = '\\';
+	    *t++ = 'r';
+	    continue;
+	  case '\f':
+	    *t++ = '\\';
+	    *t++ = 'f';
+	    continue;
 	}
 	*t++ = c;
     }
@@ -1414,15 +1472,11 @@ int
 rb_reg_options(re)
     VALUE re;
 {
-    int options = 0;
+    int options;
 
     rb_reg_check(re);
-    if (RREGEXP(re)->ptr->options & RE_OPTION_IGNORECASE)
-	options |= RE_OPTION_IGNORECASE;
-    if (RREGEXP(re)->ptr->options & RE_OPTION_MULTILINE)
-	options |= RE_OPTION_MULTILINE;
-    if (RREGEXP(re)->ptr->options & RE_OPTION_EXTENDED)
-	options |= RE_OPTION_EXTENDED;
+    options = RREGEXP(re)->ptr->options &
+	(RE_OPTION_IGNORECASE|RE_OPTION_MULTILINE|RE_OPTION_EXTENDED);
     if (FL_TEST(re, KCODE_FIXED)) {
 	options |= rb_reg_get_kcode(re);
     }
@@ -1430,7 +1484,7 @@ rb_reg_options(re)
 }
 
 static VALUE
-rb_reg_copy_object(copy, re)
+rb_reg_init_copy(copy, re)
     VALUE copy, re;
 {
     if (copy == re) return copy;
@@ -1680,7 +1734,7 @@ Init_Regexp()
     rb_define_singleton_method(rb_cRegexp, "last_match", rb_reg_s_last_match, -1);
 
     rb_define_method(rb_cRegexp, "initialize", rb_reg_initialize_m, -1);
-    rb_define_method(rb_cRegexp, "copy_object", rb_reg_copy_object, 1);
+    rb_define_method(rb_cRegexp, "initialize_copy", rb_reg_init_copy, 1);
     rb_define_method(rb_cRegexp, "hash", rb_reg_hash, 0);
     rb_define_method(rb_cRegexp, "eql?", rb_reg_equal, 1);
     rb_define_method(rb_cRegexp, "==", rb_reg_equal, 1);
@@ -1706,16 +1760,17 @@ Init_Regexp()
     rb_define_alloc_func(rb_cMatch, match_alloc);
     rb_undef_method(CLASS_OF(rb_cMatch), "new");
 
-    rb_define_method(rb_cMatch, "copy_object", match_copy_object, 1);
+    rb_define_method(rb_cMatch, "initialize_copy", match_init_copy, 1);
     rb_define_method(rb_cMatch, "size", match_size, 0);
     rb_define_method(rb_cMatch, "length", match_size, 0);
     rb_define_method(rb_cMatch, "offset", match_offset, 1);
     rb_define_method(rb_cMatch, "begin", match_begin, 1);
     rb_define_method(rb_cMatch, "end", match_end, 1);
     rb_define_method(rb_cMatch, "to_a", match_to_a, 0);
-    rb_define_method(rb_cMatch, "to_ary", match_to_a, 0);
     rb_define_method(rb_cMatch, "[]", match_aref, -1);
+    rb_define_method(rb_cMatch, "captures", match_captures, 0);
     rb_define_method(rb_cMatch, "select", match_select, -1);
+    rb_define_method(rb_cMatch, "values_at", match_values_at, -1);
     rb_define_method(rb_cMatch, "pre_match", rb_reg_match_pre, 0);
     rb_define_method(rb_cMatch, "post_match", rb_reg_match_post, 0);
     rb_define_method(rb_cMatch, "to_s", match_to_s, 0);
